[![CircleCI](https://circleci.com/gh/ThinkBigAnalytics/AoaPythonClient.svg?style=svg&circle-token=15915df83a6183fa3e0fd86ed21b1d9b91abfd29)](https://circleci.com/gh/ThinkBigAnalytics/AoaPythonClient)
- [AnalyticOps Accelerator Python Client](#analyticops-accelerator-python-client)
  - [Requirements](#requirements)
  - [Installation](#installation)
  - [Usage](#usage)
    - [Use AOA CLI](#use-aoa-cli)
      - [Add Model](#add-model)
      - [Train Model](#train-model)
      - [Evaluate Model](#evaluate-model)
    - [Configure AoaClient](#configure-aoaclient)
      - [With YAML conf](#with-yaml-conf)
      - [With env variables](#with-env-variables)
      - [With arguments](#with-arguments)
      - [Examples](#examples)
    - [Configure ModelUtility](#configure-modelutility)
      - [Examples](#examples-1)
  - [Deactivating Python environments](#deactivating-python-environments)


# AnalyticOps Accelerator Python Client

Python client for Teradata AnalyticOps Accelerator. It can be used to access AoaCoreService APIs, add a new model to git, train and evaluate model locally (without AoaCoreService).

## Requirements

Python 3.6+

## Installation

First of all, it is advisable to create a Python venv or a Conda environment to install there a
Jupyter Lab and the `aoa-python-client` Python library

Python venv:
```
python -m venv aoa_python_env
source activate aoa_python_env/bin/activate
pip install jupyterlab
```

Conda environment:
```
conda create -n aoa_python_env -q -y python=3.6
conda activate aoa_python_env
pip install jupyterlab
```

Install library from local folder using pip:

```
pip install --upgrade .
```

Install library from package file

```
# first create the package
python setup.py sdist --formats=gztar

# install using pip
pip install /path/to/tar.gz
```

## Usage

First of all you need `AOACoreService` running. Then activate the python environment you previously created and launch the Jupyter Lab server

```
jupyterlab
```

It will automatically open a new browser tab in `localhost:8888` so you can start using the Aoa framework from a
Jupyter Notebook

### Use AOA CLI

Following are the CLI options

1. Add Model
2. Train Model
3. Evaluate Model

#### Add Model

Following is the command to add model
```console
# aoa --add
model name: my first aoa model
model description: my first aoa model
These languages are supported: R, python, sql
model language: python
templates available for python: empty, pyspark, sklearn
template type (leave blank for the default one): sklearn
INFO:root:creating model structure for model: 92c713fb-0924-43fb-9a4a-f87924d8506c
```

#### Train Model

```console
# aoa --run -id 03c9a01f-bd46-4e7c-9a60-4282039094e6 -m train -d ./model_definitions/03c9a01f-bd46-4e7c-9a60-4282039094e6/.cli/datasets/train.json
Starting training...
Finished training
Saved trained model
```

#### Evaluate Model

```console
# aoa --run -id 03c9a01f-bd46-4e7c-9a60-4282039094e6 -m evaluate -d ./model_definitions/03c9a01f-bd46-4e7c-9a60-4282039094e6/.cli/datasets/train.json
model accuracy is  0.96
Evaluation complete...
```

### Configure AoaClient

Following are some code snippets to configure AoaClient

#### With YAML conf

There is an option to have the init parameters in a yaml file. For instance, these are the params used to instantiate the local client above:

```
# local_aoa_api.yml
project_id: "23e1df4b-b630-47a1-ab80-7ad5385fcd8d"
auth_user: "admin"
auth_pass: "admin"
aoa_url: "http://localhost:8080"
```

Now just pass the YAML path to the AoaClient

```python
from aoa import AoaClient
client = AoaClient(conf_file="/path/to/conf/file")
```

#### With env variables

There is an option to have the init parameters via environment variables

```python
import os
from aoa import AoaClient

os.environ["AOA_PROJECT_ID"] = "xxxx"
os.environ["AOA_API_AUTH_USER"] = "user"
os.environ["AOA_API_AUTH_PASS"] = "some_pass"
os.environ["AOA_URL"] = "http://localhost:8080"

client = AoaClient(parse_env_variables=True)
```

#### With arguments

There is an option to have the init parameters via `**kwargs`

```python
from aoa import AoaClient

client = AoaClient(project_id="xxx"``
                       auth_user="user",
                       auth_pass="some_pass",
                       aoa_url="http://localhost:8080")
```

See that in all the cases you can simply switch the project id executing the following statement,
which will change the default headers sent to the different endpoints:

```python
another_project_id = "xxxx"
client.set_project_id(another_project_id)
```

#### Examples

Following are some of the examples using AoaClient

1. [create a new project](examples/create_new_project.ipynb)
2. [update an existing project](examples/update_project.ipynb)
3. [register a dataset](examples/sample.ipynb)
4. [train, evaluate, deploy and retire a model](examples/model_lifecycle.ipynb)
5. [events of a trained model](examples/events.ipynb)

### Configure ModelUtility

Following id code snippet to configure ModelUtility

```python
from aoa import ModelUtility
# model_catalog_path is path where model are stored
utility = ModelUtility(model_catalog_path="/path/to/models/")
```
Note: model_catalog_path is the base path of model directory. In the following example `/Users/ec2-user/Work/models` is model_catalog_path

```console
/Users/ec2-user/Work/models
├── local_aoa_api.yml
├── model_definitions
│   └── 4a233000-7eea-4739-ae32-c2b4329f6b75
│       ├── README.md
│       ├── config.json
│       ├── model.json
│       ├── model_modules
│       │   ├── __init__.py
│       │   ├── __pycache__
│       │   │   ├── __init__.cpython-37.pyc
│       │   │   ├── scoring.cpython-37.pyc
│       │   │   └── training.cpython-37.pyc
│       │   ├── requirements.txt
│       │   ├── scoring.py
│       │   └── training.py
│       └── notebooks
│           └── Explore\ Diabetes.ipynb
```

#### Examples

Following are some of the examples using ModelUtility

1. [add a new model](examples/add_new_model.ipynb)
2. [train and evaluate a model locally](examples/train_evaluate_model_locally.ipynb)


## Deactivating Python environments

venv:
```
deactivate
```

conda environment:
```
conda deactivate
```