from aoa.model_utility.model_utility import ModelUtility
import logging
import json
import os
from jinja2 import Template


class BaseModel(object):

    def __init__(self, model_utility: ModelUtility):
        self.model_utility = model_utility

    @staticmethod
    def get_dataset_metadata(model_dir, rtn_val=False):
        catalog = {}
        index = 0
        dataset_ids = "Use one of the following datasets\n"
        base_path = os.path.join(model_dir, '') + ".cli/datasets/"

        if not os.path.exists(base_path):
            raise Exception(
                "No .cli/datasets exist in model folder. You must specify dataset path via argument")

        for filename in os.listdir(base_path):
            with open(base_path + filename, 'r') as f:
                catalog[index] = filename
                index += 1

        if rtn_val:
            return catalog
        else:
            if len(catalog) == 1:
                return base_path + catalog[int(0)]
            elif len(catalog) == 0:
                raise Exception("no dataset found")
            else:
                for key in catalog:
                    dataset_ids += "{1}: {0}\n".format(key, catalog[key])
                raise ValueError(dataset_ids)

    @staticmethod
    def get_model_id(model_path, rtn_val=False):
        catalog = {}
        index = 0
        model_ids = "Use one of the following models\n"

        for model in os.listdir(model_path):
            if os.path.exists(model_path + model + "/model.json"):
                with open(model_path + model + "/model.json", 'r') as f:
                    model_definition = json.load(f)
                    catalog[index] = model_definition
                    index += 1

        for key in catalog:
            model_ids += "{1}: {0}\n".format(catalog[key]["id"], catalog[key]["name"])

        if rtn_val:
            return catalog

        raise ValueError(model_ids)

    def __template_sql_script(self, filename, jinja_ctx):
        with open(filename) as f:
            template = Template(f.read())

        return template.render(jinja_ctx)

    def __execute_sql_script(self, conn, filename, jinja_ctx):
        script = self.__template_sql_script(filename, jinja_ctx)

        stms = script.split(';')

        for stm in stms:
            stm = stm.strip()
            if stm:
                logging.info("Executing statement: {}".format(stm))

                try:
                    conn.execute(stm)
                except Exception as e:
                    if stm.startswith("DROP"):
                        logging.warning("Ignoring DROP statement exception")
                    else:
                        raise e
