from aoa.model_utility.base_model import BaseModel
import json
import os
import subprocess
import sys


class EvaluateModel(BaseModel):

    def evaluate_model_local(self, model_id: str = None, base_path: str = None, dataset_path: str = None):
        base_path = self.model_utility.model_catalog_path if base_path is None else os.path.join(base_path, '')
        model_path = base_path + 'model_definitions/'

        if not os.path.exists(model_path):
            raise ValueError("model directory {0} does not exist".format(model_path))

        if model_id is None:
            if "id" in self.model_utility.model:
                model_id = self.model_utility.model["id"]
            else:
                BaseModel.get_model_id(model_path)

        model_dir = model_path + model_id

        cli_model_kargs = {
            "model_id": model_id,
            "model_version": "cli",
            "model_table": "AOA_MODELS_cli"
        }

        if dataset_path is None:
            dataset_path = BaseModel.get_dataset_metadata(model_dir)

        with open(dataset_path, 'r') as f:
            data_conf = json.load(f)

        with open(model_dir + "/model.json", 'r') as f:
            model_definition = json.load(f)

        with open(model_dir + "/config.json", 'r') as f:
            model_conf = json.load(f)

        if model_definition["language"] == "python":
            sys.path.append(model_dir)
            import model_modules
            model_modules.scoring.evaluate(data_conf, model_conf, **cli_model_kargs)
        elif model_definition["language"] == "sql":
            self.__evaluate_sql(model_dir, data_conf, model_conf, **cli_model_kargs)
        elif model_definition["language"] == "R":
            cmd = self.model_utility.dir_path + "/run_model.R {} {} {} {}".format(model_id, "evaluate", dataset_path, base_path)
            subprocess.check_call(cmd, shell=True)
        else:
            raise Exception("Unsupported language: {}".format(model_definition["language"]))

    def __evaluate_sql(self, model_dir, data_conf, model_conf, **kwargs):
        from teradataml import create_context
        from teradataml.dataframe.dataframe import DataFrame
        from teradataml.context.context import get_connection

        create_context(host=data_conf["hostname"],
                       username=os.environ["TD_USERNAME"],
                       password=os.environ["TD_PASSWORD"],
                       logmech=os.getenv("TD_LOGMECH", "TDNEGO"))

        print("Starting evaluation...")

        sql_file = model_dir + "/model_modules/scoring.sql"
        jinja_ctx = {
            "data_conf": data_conf,
            "model_conf": model_conf,
            "model_table": kwargs.get("model_table"),
            "model_version": kwargs.get("model_version"),
            "model_id": kwargs.get("model_id")
        }

        self._BaseModel__execute_sql_script(get_connection(), sql_file, jinja_ctx)

        print("Finished evaluation")

        stats = DataFrame(data_conf["metrics_table"]).to_pandas()
        metrics = dict(zip(stats.key, stats.value))

        with open("models/evaluation.json", "w+") as f:
            json.dump(metrics, f)
