from aoa.model_utility.base_model import BaseModel
import shutil
import os
import json
import subprocess
import sys


class TrainModel(BaseModel):

    def train_model_local(self, model_id: str = None, base_path: str = None, dataset_path: str = None):
        base_path = self.model_utility.model_catalog_path if base_path is None else os.path.join(base_path, '')
        model_path = base_path + 'model_definitions/'

        if not os.path.exists(model_path):
            raise ValueError("model directory {0} does not exist".format(model_path))

        if model_id is None:
            if "id" in self.model_utility.model:
                model_id = self.model_utility.model["id"]
            else:
                BaseModel.get_model_id(model_path)

        if os.path.exists("models"):
            shutil.rmtree("models")
        os.makedirs("models")

        model_dir = model_path + model_id

        cli_model_kargs = {
            "model_id": model_id,
            "model_version": "cli",
            "model_table": "AOA_MODELS_cli"
        }

        if dataset_path is None:
            dataset_path = BaseModel.get_dataset_metadata(model_dir)

        with open(dataset_path, 'r') as f:
            data_conf = json.load(f)

        with open(model_dir + "/model.json", 'r') as f:
            model_definition = json.load(f)

        with open(model_dir + "/config.json", 'r') as f:
            model_conf = json.load(f)

        if model_definition["language"] == "python":
            sys.path.append(model_dir)
            import model_modules
            model_modules.training.train(data_conf, model_conf, **cli_model_kargs)
        elif model_definition["language"] == "sql":
            self.__train_sql(model_dir, data_conf, model_conf, **cli_model_kargs)
        elif model_definition["language"] == "R":
            cmd = self.model_utility.dir_path + "/run_model.R {} {} {} {}".format(model_id, "train", dataset_path, base_path)
            subprocess.check_call(cmd, shell=True)
        else:
            raise Exception("Unsupported language: {}".format(model_definition["language"]))

    def __train_sql(self, model_dir, data_conf, model_conf, **kwargs):
        from teradataml import create_context
        from teradataml.context.context import get_connection

        jinja_ctx = {
            "data_conf": data_conf,
            "model_conf": model_conf,
            "model_table": kwargs.get("model_table"),
            "model_version": kwargs.get("model_version"),
            "model_id": kwargs.get("model_id")
        }

        print("Starting training...")

        sql_file = model_dir + "training.sql"
        jinja_ctx = {"data_conf": data_conf, "model_conf": model_conf}

        self._BaseModel__execute_sql_script(get_connection(), sql_file, jinja_ctx)

        print("Finished training")

        print("Saved trained model")


